@@ -20,6 +20,7 @@
 //! [Apache Arrow]: https://arrow.apache.org
 //! [Apache Avro]: https://avro.apache.org/
 
+#![warn(missing_docs)]
 #![allow(unused)] // Temporary
 
 pub mod reader;
@@ -27,6 +28,8 @@ mod schema;
 
 mod compression;
 
+mod codec;
+
 #[cfg(test)]
 mod test_util {
     pub fn arrow_test_data(path: &str) -> String {
