@@ -15,14 +15,21 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Utilities for pretty printing record batches. Note this module is not
-//! available unless `feature = "prettyprint"` is enabled.
+//! Utilities for pretty printing [`RecordBatch`]es and [`Array`]s.
+//!
+//! Note this module is not available unless `feature = "prettyprint"` is enabled.
+//!
+//! [`RecordBatch`]: arrow_array::RecordBatch
+//! [`Array`]: arrow_array::Array
+
+use std::fmt::Display;
+
+use comfy_table::{Cell, Table};
 
-use crate::display::{ArrayFormatter, FormatOptions};
 use arrow_array::{Array, ArrayRef, RecordBatch};
 use arrow_schema::ArrowError;
-use comfy_table::{Cell, Table};
-use std::fmt::Display;
+
+use crate::display::{ArrayFormatter, FormatOptions};
 
 /// Create a visual representation of record batches
 pub fn pretty_format_batches(results: &[RecordBatch]) -> Result<impl Display, ArrowError> {
@@ -47,7 +54,8 @@ pub fn pretty_format_columns(
     pretty_format_columns_with_options(col_name, results, &options)
 }
 
-pub fn pretty_format_columns_with_options(
+/// Utility function to create a visual representation of columns with options
+fn pretty_format_columns_with_options(
     col_name: &str,
     results: &[ArrayRef],
     options: &FormatOptions,
@@ -131,17 +139,20 @@ fn create_column(
 
 #[cfg(test)]
 mod tests {
+    use std::fmt::Write;
+    use std::sync::Arc;
+
+    use half::f16;
 
-    use super::*;
-    use crate::display::array_value_to_string;
     use arrow_array::builder::*;
     use arrow_array::types::*;
     use arrow_array::*;
-    use arrow_buffer::Buffer;
+    use arrow_buffer::{IntervalDayTime, IntervalMonthDayNano, ScalarBuffer};
     use arrow_schema::*;
-    use half::f16;
-    use std::fmt::Write;
-    use std::sync::Arc;
+
+    use crate::display::array_value_to_string;
+
+    use super::*;
 
     #[test]
     fn test_pretty_format_batches() {
@@ -317,6 +328,82 @@ mod tests {
         assert_eq!(expected, actual, "Actual result:\n{table}");
     }
 
+    #[test]
+    fn test_pretty_format_string_view() {
+        let schema = Arc::new(Schema::new(vec![Field::new(
+            "d1",
+            DataType::Utf8View,
+            true,
+        )]));
+
+        // Use a small capacity so we end up with multiple views
+        let mut builder = StringViewBuilder::with_capacity(20);
+        builder.append_value("hello");
+        builder.append_null();
+        builder.append_value("longer than 12 bytes");
+        builder.append_value("another than 12 bytes");
+        builder.append_null();
+        builder.append_value("small");
+
+        let array: ArrayRef = Arc::new(builder.finish());
+        let batch = RecordBatch::try_new(schema, vec![array]).unwrap();
+        let table = pretty_format_batches(&[batch]).unwrap().to_string();
+        let expected = vec![
+            "+-----------------------+",
+            "| d1                    |",
+            "+-----------------------+",
+            "| hello                 |",
+            "|                       |",
+            "| longer than 12 bytes  |",
+            "| another than 12 bytes |",
+            "|                       |",
+            "| small                 |",
+            "+-----------------------+",
+        ];
+
+        let actual: Vec<&str> = table.lines().collect();
+
+        assert_eq!(expected, actual, "Actual result:\n{table:#?}");
+    }
+
+    #[test]
+    fn test_pretty_format_binary_view() {
+        let schema = Arc::new(Schema::new(vec![Field::new(
+            "d1",
+            DataType::BinaryView,
+            true,
+        )]));
+
+        // Use a small capacity so we end up with multiple views
+        let mut builder = BinaryViewBuilder::with_capacity(20);
+        builder.append_value(b"hello");
+        builder.append_null();
+        builder.append_value(b"longer than 12 bytes");
+        builder.append_value(b"another than 12 bytes");
+        builder.append_null();
+        builder.append_value(b"small");
+
+        let array: ArrayRef = Arc::new(builder.finish());
+        let batch = RecordBatch::try_new(schema, vec![array]).unwrap();
+        let table = pretty_format_batches(&[batch]).unwrap().to_string();
+        let expected = vec![
+            "+--------------------------------------------+",
+            "| d1                                         |",
+            "+--------------------------------------------+",
+            "| 68656c6c6f                                 |",
+            "|                                            |",
+            "| 6c6f6e676572207468616e203132206279746573   |",
+            "| 616e6f74686572207468616e203132206279746573 |",
+            "|                                            |",
+            "| 736d616c6c                                 |",
+            "+--------------------------------------------+",
+        ];
+
+        let actual: Vec<&str> = table.lines().collect();
+
+        assert_eq!(expected, actual, "Actual result:\n\n{table:#?}");
+    }
+
     #[test]
     fn test_pretty_format_fixed_size_binary() {
         // define a schema.
@@ -769,14 +856,18 @@ mod tests {
 
         // Can't use UnionBuilder with non-primitive types, so manually build outer UnionArray
         let a_array = Int32Array::from(vec![None, None, None, Some(1234), Some(23)]);
-        let type_ids = Buffer::from_slice_ref([1_i8, 1, 0, 0, 1]);
+        let type_ids = [1, 1, 0, 0, 1].into_iter().collect::<ScalarBuffer<i8>>();
 
-        let children: Vec<(Field, Arc<dyn Array>)> = vec![
-            (Field::new("a", DataType::Int32, true), Arc::new(a_array)),
-            (inner_field.clone(), Arc::new(inner)),
-        ];
+        let children = vec![Arc::new(a_array) as Arc<dyn Array>, Arc::new(inner)];
 
-        let outer = UnionArray::try_new(&[0, 1], type_ids, None, children).unwrap();
+        let union_fields = [
+            (0, Arc::new(Field::new("a", DataType::Int32, true))),
+            (1, Arc::new(inner_field.clone())),
+        ]
+        .into_iter()
+        .collect();
+
+        let outer = UnionArray::try_new(union_fields, type_ids, None, children).unwrap();
 
         let schema = Schema::new(vec![Field::new_union(
             "Teamsters",
@@ -877,12 +968,12 @@ mod tests {
     #[test]
     fn test_pretty_format_interval_day_time() {
         let arr = Arc::new(arrow_array::IntervalDayTimeArray::from(vec![
-            Some(-600000),
-            Some(4294966295),
-            Some(4294967295),
-            Some(1),
-            Some(10),
-            Some(100),
+            Some(IntervalDayTime::new(-1, -600_000)),
+            Some(IntervalDayTime::new(0, -1001)),
+            Some(IntervalDayTime::new(0, -1)),
+            Some(IntervalDayTime::new(0, 1)),
+            Some(IntervalDayTime::new(0, 10)),
+            Some(IntervalDayTime::new(0, 100)),
         ]));
 
         let schema = Arc::new(Schema::new(vec![Field::new(
@@ -896,16 +987,16 @@ mod tests {
         let table = pretty_format_batches(&[batch]).unwrap().to_string();
 
         let expected = vec![
-            "+----------------------------------------------------+",
-            "| IntervalDayTime                                    |",
-            "+----------------------------------------------------+",
-            "| 0 years 0 mons -1 days 0 hours -10 mins 0.000 secs |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins -1.001 secs   |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins -0.001 secs   |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.001 secs    |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.010 secs    |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.100 secs    |",
-            "+----------------------------------------------------+",
+            "+------------------+",
+            "| IntervalDayTime  |",
+            "+------------------+",
+            "| -1 days -10 mins |",
+            "| -1.001 secs      |",
+            "| -0.001 secs      |",
+            "| 0.001 secs       |",
+            "| 0.010 secs       |",
+            "| 0.100 secs       |",
+            "+------------------+",
         ];
 
         let actual: Vec<&str> = table.lines().collect();
@@ -916,19 +1007,19 @@ mod tests {
     #[test]
     fn test_pretty_format_interval_month_day_nano_array() {
         let arr = Arc::new(arrow_array::IntervalMonthDayNanoArray::from(vec![
-            Some(-600000000000),
-            Some(18446744072709551615),
-            Some(18446744073709551615),
-            Some(1),
-            Some(10),
-            Some(100),
-            Some(1_000),
-            Some(10_000),
-            Some(100_000),
-            Some(1_000_000),
-            Some(10_000_000),
-            Some(100_000_000),
-            Some(1_000_000_000),
+            Some(IntervalMonthDayNano::new(-1, -1, -600_000_000_000)),
+            Some(IntervalMonthDayNano::new(0, 0, -1_000_000_001)),
+            Some(IntervalMonthDayNano::new(0, 0, -1)),
+            Some(IntervalMonthDayNano::new(0, 0, 1)),
+            Some(IntervalMonthDayNano::new(0, 0, 10)),
+            Some(IntervalMonthDayNano::new(0, 0, 100)),
+            Some(IntervalMonthDayNano::new(0, 0, 1_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 10_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 100_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 1_000_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 10_000_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 100_000_000)),
+            Some(IntervalMonthDayNano::new(0, 0, 1_000_000_000)),
         ]));
 
         let schema = Arc::new(Schema::new(vec![Field::new(
@@ -942,23 +1033,23 @@ mod tests {
         let table = pretty_format_batches(&[batch]).unwrap().to_string();
 
         let expected = vec![
-            "+-----------------------------------------------------------+",
-            "| IntervalMonthDayNano                                      |",
-            "+-----------------------------------------------------------+",
-            "| 0 years -1 mons -1 days 0 hours -10 mins 0.000000000 secs |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins -1.000000001 secs    |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins -0.000000001 secs    |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000000001 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000000010 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000000100 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000001000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000010000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.000100000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.001000000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.010000000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 0.100000000 secs     |",
-            "| 0 years 0 mons 0 days 0 hours 0 mins 1.000000000 secs     |",
-            "+-----------------------------------------------------------+",
+            "+--------------------------+",
+            "| IntervalMonthDayNano     |",
+            "+--------------------------+",
+            "| -1 mons -1 days -10 mins |",
+            "| -1.000000001 secs        |",
+            "| -0.000000001 secs        |",
+            "| 0.000000001 secs         |",
+            "| 0.000000010 secs         |",
+            "| 0.000000100 secs         |",
+            "| 0.000001000 secs         |",
+            "| 0.000010000 secs         |",
+            "| 0.000100000 secs         |",
+            "| 0.001000000 secs         |",
+            "| 0.010000000 secs         |",
+            "| 0.100000000 secs         |",
+            "| 1.000000000 secs         |",
+            "+--------------------------+",
         ];
 
         let actual: Vec<&str> = table.lines().collect();
