@@ -69,20 +69,8 @@
 pub mod reader;
 pub mod writer;
 
-#[doc(hidden)]
-#[deprecated(note = "Use Decoder")]
-pub type RawDecoder = reader::Decoder;
-
-#[doc(hidden)]
-#[deprecated(note = "Use Reader")]
-pub type RawReader<R> = Reader<R>;
-
-#[doc(hidden)]
-#[deprecated(note = "Use ReaderBuilder")]
-pub type RawReaderBuilder = ReaderBuilder;
-
 pub use self::reader::{Reader, ReaderBuilder};
-pub use self::writer::{ArrayWriter, LineDelimitedWriter, Writer};
+pub use self::writer::{ArrayWriter, LineDelimitedWriter, Writer, WriterBuilder};
 use half::f16;
 use serde_json::{Number, Value};
 
@@ -144,10 +132,7 @@ impl JsonSerializable for f64 {
 mod tests {
     use super::*;
 
-    use serde_json::{
-        Number,
-        Value::{Bool, Number as VNumber, String as VString},
-    };
+    use serde_json::Value::{Bool, Number as VNumber, String as VString};
 
     #[test]
     fn test_arrow_native_type_to_json() {
