@@ -122,10 +122,8 @@ fn add_benchmark(c: &mut Criterion) {
         b.iter(|| bench_sort_to_indices(&arr, None))
     });
 
-    let run_encoded_array = create_primitive_run_array::<Int16Type, Int32Type>(
-        2usize.pow(12),
-        2usize.pow(10),
-    );
+    let run_encoded_array =
+        create_primitive_run_array::<Int16Type, Int32Type>(2usize.pow(12), 2usize.pow(10));
 
     c.bench_function("sort primitive run 2^12", |b| {
         b.iter(|| bench_sort(&run_encoded_array))
