@@ -75,6 +75,7 @@ fn is_leaf(data_type: &DataType) -> bool {
             | DataType::Float32
             | DataType::Float64
             | DataType::Utf8
+            | DataType::Utf8View
             | DataType::LargeUtf8
             | DataType::Timestamp(_, _)
             | DataType::Date32
@@ -85,6 +86,7 @@ fn is_leaf(data_type: &DataType) -> bool {
             | DataType::Interval(_)
             | DataType::Binary
             | DataType::LargeBinary
+            | DataType::BinaryView
             | DataType::Decimal128(_, _)
             | DataType::Decimal256(_, _)
             | DataType::FixedSizeBinary(_)
@@ -101,6 +103,7 @@ struct LevelContext {
 }
 
 /// A helper to construct [`ArrayLevels`] from a potentially nested [`Field`]
+#[derive(Debug)]
 enum LevelInfoBuilder {
     /// A primitive, leaf array
     Primitive(ArrayLevels),
@@ -132,7 +135,15 @@ enum LevelInfoBuilder {
 impl LevelInfoBuilder {
     /// Create a new [`LevelInfoBuilder`] for the given [`Field`] and parent [`LevelContext`]
     fn try_new(field: &Field, parent_ctx: LevelContext, array: &ArrayRef) -> Result<Self> {
-        assert_eq!(field.data_type(), array.data_type());
+        if field.data_type() != array.data_type() {
+            return Err(arrow_err!(format!(
+                "Incompatible type. Field '{}' has type {}, array has type {}",
+                field.name(),
+                field.data_type(),
+                array.data_type(),
+            )));
+        }
+
         let is_nullable = field.is_nullable();
 
         match array.data_type() {
@@ -608,10 +619,7 @@ impl ArrayLevels {
 mod tests {
     use super::*;
 
-    use std::sync::Arc;
-
     use arrow_array::builder::*;
-    use arrow_array::cast::AsArray;
     use arrow_array::types::Int32Type;
     use arrow_array::*;
     use arrow_buffer::{Buffer, ToByteSlice};
@@ -1042,7 +1050,7 @@ mod tests {
         let a_list_data = ArrayData::builder(a_list_type.clone())
             .len(5)
             .add_buffer(a_value_offsets)
-            .null_bit_buffer(Some(Buffer::from(vec![0b00011011])))
+            .null_bit_buffer(Some(Buffer::from([0b00011011])))
             .add_child_data(a_values.to_data())
             .build()
             .unwrap();
@@ -1058,7 +1066,7 @@ mod tests {
 
         assert_eq!(levels.len(), 1);
 
-        let list_level = levels.get(0).unwrap();
+        let list_level = &levels[0];
 
         let expected_level = ArrayLevels {
             def_levels: Some(vec![0, 3, 3, 3]),
@@ -1108,7 +1116,7 @@ mod tests {
 
         // Construct a buffer for value offsets, for the nested array:
         //  [[1], [2, 3], null, [4, 5, 6], [7, 8, 9, 10]]
-        let g_value_offsets = arrow::buffer::Buffer::from(&[0, 1, 3, 3, 6, 10].to_byte_slice());
+        let g_value_offsets = arrow::buffer::Buffer::from([0, 1, 3, 3, 6, 10].to_byte_slice());
 
         // Construct a list array from the above two
         let g_list_data = ArrayData::builder(struct_field_g.data_type().clone())
@@ -1150,7 +1158,7 @@ mod tests {
         assert_eq!(levels.len(), 5);
 
         // test "a" levels
-        let list_level = levels.get(0).unwrap();
+        let list_level = &levels[0];
 
         let expected_level = ArrayLevels {
             def_levels: None,
@@ -1295,7 +1303,7 @@ mod tests {
         let map = batch.column(0).as_map();
 
         // test key levels
-        let list_level = levels.get(0).unwrap();
+        let list_level = &levels[0];
 
         let expected_level = ArrayLevels {
             def_levels: Some(vec![1; 7]),
@@ -1477,9 +1485,10 @@ mod tests {
         let field_a2 = Arc::new(Field::new("integers", a2.data_type().clone(), true));
 
         let nulls = Buffer::from([0b00110111]);
-        let struct_a =
-            Arc::new(StructArray::try_from((vec![(field_a1, a1), (field_a2, a2)], nulls)).unwrap())
-                as ArrayRef;
+        let struct_a = Arc::new(StructArray::from((
+            vec![(field_a1, a1), (field_a2, a2)],
+            nulls,
+        ))) as ArrayRef;
 
         let offsets = Buffer::from_iter([0_i32, 0, 2, 2, 3, 5, 5]);
         let nulls = Buffer::from([0b00111100]);
@@ -1566,7 +1575,7 @@ mod tests {
 
         assert_eq!(levels.len(), 1);
 
-        let list_level = levels.get(0).unwrap();
+        let list_level = &levels[0];
 
         let expected_level = ArrayLevels {
             def_levels: Some(vec![0, 0, 3, 3]),
@@ -1756,7 +1765,7 @@ mod tests {
 
         assert_eq!(levels.len(), 1);
 
-        let list_level = levels.get(0).unwrap();
+        let list_level = &levels[0];
 
         let expected_level = ArrayLevels {
             def_levels: Some(vec![1, 0, 1]),
@@ -1834,6 +1843,21 @@ mod tests {
         assert_eq!(levels[0], expected_level);
     }
 
+    #[test]
+    fn mismatched_types() {
+        let array = Arc::new(Int32Array::from_iter(0..10)) as ArrayRef;
+        let field = Field::new("item", DataType::Float64, false);
+
+        let err = LevelInfoBuilder::try_new(&field, Default::default(), &array)
+            .unwrap_err()
+            .to_string();
+
+        assert_eq!(
+            err,
+            "Arrow: Incompatible type. Field 'item' has type Float64, array has type Int32",
+        );
+    }
+
     fn levels<T: Array + 'static>(field: &Field, array: T) -> LevelInfoBuilder {
         let v = Arc::new(array) as ArrayRef;
         LevelInfoBuilder::try_new(field, Default::default(), &v).unwrap()
